#version 330 core

//this is the final rendering of the atmosphere
//it completes the planet view

//for more information see:
//Real Time Atmosphere Rendering for the Space Simulators
//by Radovan Josth

//input data: (from vertex shader)
in vec2 UV;
in vec3 pos_worldspace;
in vec3 normal_cameraspace;
in vec3 eyeDirection_cameraspace;
in vec3 lightDirection_cameraspace;
in vec4 gl_FragCoord;

//output data:
//out vec3 color; //this is normally correct -> but
layout(location = 0) out vec3 color; //we are using DOF which itself renders to a texture

//values that stay constant for the whole mesh:
uniform mat4 MV;
uniform vec3 lightPos_worldspace;
uniform sampler2D textureShadowBack;
uniform sampler2D textureShadowFront;
uniform sampler2D textureAtmosDepth;
uniform sampler2D texturePlanetDepth;
//uniform sampler2DMS texturePlanetColor;
uniform sampler2D texturePlanetColor;
uniform vec2 screenSize;

float zNear = 0.01;
float zFar  = 1000000.0;

vec4 linearizeDepth( vec4 depthValue )
{
	float a = zFar / (zFar - zNear);
	float b = zFar * zNear / (zNear - zFar);
	return b / (depthValue - a);
}

void main()
{
	float zBack = texture2D( textureAtmosDepth, gl_FragCoord.xy/screenSize ).x;
	float zshb  = min( max(texture2D(textureShadowBack, gl_FragCoord.xy/screenSize).x, gl_FragCoord.z), zBack );
	float zshf  = min( max(texture2D(textureShadowFront, gl_FragCoord.xy/screenSize).x, gl_FragCoord.z), zBack );
	
	//linearize z values:
	vec4 z = vec4( zBack, zshb, zshf, gl_FragCoord.z );
	z = linearizeDepth( z ); //zLin = b/(z-a)
	
	//dz.x => width of the illuminated atmosphere
	//dz.y => width of the shadowed atmosphere
	vec2 dz = vec2( z.x-z.y+z.z-z.w, z.y-z.z );
	
	//stretch dz to correct depth ratio:
	//dz = dz / (zFar - zNear); //not needed
	
	/*vec3 weightedColor = vec3(0,0,0);
	for ( int i=0; i<4; i++ )
	{
		vec3 sample    = texelFetch(texturePlanetColor, ivec2(floor(gl_FragCoord.xy)), i).rgb;
		weightedColor += sample/4.0;
	}*/
	
	//3.0 ... atmosphere brightness
	//20% ... color of the shadowed atmosphere
	//color.rgb = weightedColor + 2.0*( vec3(0,0,1)*(dz.x + 0.2*dz.y) );
	color.rgb = texture2D(texturePlanetColor, gl_FragCoord.xy/screenSize).rgb + 2.3*( vec3(0,0,1)*(dz.x + 0.2*dz.y) );
}
