#version 330 core

//this is for the clouds -> so no specular term!

//mostly taken from:
//http://code.google.com/p/opengl-tutorial-org/source/browse/tutorial08_basic_shading/StandardShading.fragmentshader

//input data: (from vertex shader)
in vec2 UV;
in vec3 pos_worldspace;
in vec3 normal_cameraspace;
in vec3 eyeDirection_cameraspace;
in vec3 lightDirection_cameraspace;

//output data:
layout(location=0) out vec3 color;

//values that stay constant for the whole mesh:
uniform sampler2D diffuseTexture;
uniform mat4 MV;
uniform vec3 lightPos_worldspace;

void main()
{
	//light emission properties (TODO: maybe change this into uniforms)
	vec3  lightColor = vec3( 1, 1, 1 );
	float lightPower = 80000.0f;
	
	//material properties:
	vec3 materialDiffuseColor  = texture2D( diffuseTexture, UV ).rgb;
	vec3 materialAmbientColor  = vec3(0.1, 0.1, 0.1) * materialDiffuseColor;
	
	//distance to the light:
	float distance = length( lightPos_worldspace - pos_worldspace );
	
	//normal of the computed fragment, in camera space:
	vec3 n = normalize( normal_cameraspace );
	
	//direction from fragment to light:
	vec3 l = normalize( lightDirection_cameraspace );
	
	//diffuse lightning: cosine of angle between normal and light direction:
	//clamped below 0 and above 1
	//  - light is at the vertical of the triangle -> 1
	//  - light is perpendicular to the triangle -> 0
	//  - light is behind the triangle -> 0
	float cosTheta = clamp( dot(n, l), 0, 1 );
	
	color = materialAmbientColor + //indirect lighting
			//color of the object (with respect to where lightsource is)
			materialDiffuseColor  * lightColor * lightPower * cosTheta/(distance); //normally /distance*distance -> but sun is far away!
}
