#version 330 core

//mostly taken from:
//http://code.google.com/p/opengl-tutorial-org/source/browse/tutorial08_basic_shading/StandardShading.fragmentshader

//input data: (from vertex shader)
in vec2 UV;
in vec3 pos_worldspace;
in vec3 normal_cameraspace;
in vec3 eyeDirection_cameraspace;
in vec3 lightDirection_cameraspace;

//output data:
layout(location=0) out vec3 color;

//values that stay constant for the whole mesh:
uniform sampler2D diffuseTexture;
uniform sampler2D specularTexture;
uniform mat4 MV;
uniform vec3 lightPos_worldspace;
uniform vec2 validityChecker; //if part one has a value different than 0, this has a specularTexture. y-component is not yet used -> intendet for future use of normal map...

void main()
{
	//light emission properties (TODO: maybe change this into uniforms)
	vec3  lightColor = vec3( 1, 1, 1 );
	float lightPower = 80000.0f;
	
	//material properties:
	vec3 materialDiffuseColor  = texture2D( diffuseTexture, UV ).rgb;
	vec3 materialAmbientColor  = vec3(0.1, 0.1, 0.1) * materialDiffuseColor;
	/*vec3 materialSpecularColor;
	if ( validityChecker.x != 0 )
	{
		//fake a specular map -> better results than with having one!
		if ( materialDiffuseColor.z > 0.4 && materialDiffuseColor.x < 0.1 )
			materialSpecularColor = vec3(materialDiffuseColor.z*0.5, materialDiffuseColor.z*0.5, materialDiffuseColor.z*0.5);
		else
			materialSpecularColor = vec3(0,0,0);
	} else
	{
		materialSpecularColor = vec3(0,0,0);
	}*/
	vec3 materialSpecularColor = validityChecker.x == 0 ? vec3(0.3, 0.3, 0.3) : texture2D( specularTexture, UV ).rgb;
	
	//distance to the light:
	float distance = length( lightPos_worldspace - pos_worldspace );
	
	//normal of the computed fragment, in camera space:
	vec3 n = normalize( normal_cameraspace );
	
	//direction from fragment to light:
	vec3 l = normalize( lightDirection_cameraspace );
	
	//diffuse lightning: cosine of angle between normal and light direction:
	//clamped below 0 and above 1
	//  - light is at the vertical of the triangle -> 1
	//  - light is perpendicular to the triangle -> 0
	//  - light is behind the triangle -> 0
	float cosTheta = clamp( dot(n, l), 0, 1 );
	
	//eye vector (towards the camera):
	vec3 eye = normalize( eyeDirection_cameraspace );
	
	//direction in which the triangle reflects the light:
	vec3 reflection = reflect( -l, n );
	
	//specular reflection: cosine of angle between eye vector and reflect vector:
	//clamped below 0 and above 1:
	//  - looking into the reflection -> 1
	//  - looking elsewhere -> -1
	float cosAlpha = clamp( dot(eye, reflection), 0, 1 );

		
	color = materialAmbientColor + //indirect lighting
			//color of the object (with respect to where lightsource is)
			materialDiffuseColor  * lightColor * lightPower * cosTheta/(distance) + //normally /distance*distance -> but sun is far away!
			//reflective highlight (respect lightsource and lookDirection)
			materialSpecularColor * lightColor * lightPower * pow( cosAlpha, 5 ) / (distance); //normally /distance*distance -> but sun is far away!
}
