#version 330 core

//mostly taken from:
//http://code.google.com/p/opengl-tutorial-org/source/browse/tutorial08_basic_shading/StandardShading.vertexshader

//input vertex data, different for all executions of this shader:
layout(location = 0) in vec3 vertexPos_modelspace;
layout(location = 1) in vec2 vertexUV;
layout(location = 2) in vec3 vertexNormal_modelspace;

//output data: will be interpolated for each fragment:
out vec2 UV;
out vec3 pos_worldspace;
out vec3 normal_cameraspace;
out vec3 eyeDirection_cameraspace;
out vec3 lightDirection_cameraspace;

//values that stay constant for the whole mesh:
uniform mat4 MVP;
uniform mat4 V;
uniform mat4 M;
uniform vec3 lightPos_worldspace;

void main()
{
	//output position of the vertex, in clip space : MVP * position
	gl_Position = MVP * vec4( vertexPos_modelspace, 1 );
	
	//vertex pos in worldspace:
	pos_worldspace = (M * vec4(vertexPos_modelspace, 1)).xyz;
	
	//vector from vertex to light (in cameraspace):
	//in cameraspace camera is at position 0,0,0:
	vec3 vertexPos_cameraspace = (V * M * vec4(vertexPos_modelspace, 1)).xyz;
	eyeDirection_cameraspace   = vec3(0,0,0) - vertexPos_cameraspace;
	
	//vector from vertex to light (in cameraspae).
	vec3 lightPos_cameraspace  = (V * vec4(lightPos_worldspace, 1)).xyz;
	lightDirection_cameraspace = lightPos_cameraspace - vertexPos_cameraspace;
	
	//normal of the vertex, in cameraspace:
	normal_cameraspace = (V * M * vec4(vertexNormal_modelspace,0)).xyz; // Only correct if ModelMatrix does not scale the model ! Use its inverse transpose if not.
	
	//UV of the vertex:
	UV = vertexUV;
}
